@@ -88,7 +88,15 @@ mod test_binary_kinds {
         case("2 % 1"),
         case("'hello' + \"world\""),
         case("2 * bar"),
-        case("2 / foo")
+        case("2 / foo"),
+        case("true & false"),
+        case("15 & 20"),
+        case("true | false"),
+        case("-8 | 23"),
+        case("true ^ false"),
+        case("2 ^ 12"),
+        case("2 << 8"),
+        case("300 >> 3")
     )]
     fn test(expr: &str) {
         check_scalar_evaluation(expr, &config_a())
@@ -175,6 +183,7 @@ mod test_ternary {
         case("1 + (-1? 8: 23)"),
         case("1? 2: 3? 4: 5? 6: 7"),
         case("(1? 2: (3? 4: (5? 6: 7)))"),
+        case("(0? 2: (false? 4: (5? 6: 7)))"),
         case("(((1? 2: 3)? 4: 5)? 6: 7)")
     )]
     fn test(expr: &str) {
