@@ -141,7 +141,10 @@ mod test_custom_specs {
         case("custom/gh_361", 0.001, true),
         case("custom/gh_379", 0.001, true),
         case("custom/gh_383", 0.001, true),
-        case("custom/gh_391", 0.001, true)
+        case("custom/gh_391", 0.001, true),
+        case("custom/facet_grouped_bar_with_error_bars", 0.001, true),
+        case("custom/facet_grouped_bar_with_error_bars_with_sort", 0.001, true),
+        case("custom/binned_ordinal", 0.001, true)
     )]
     fn test_image_comparison(spec_name: &str, tolerance: f64, extract_inline_values: bool) {
         println!("spec_name: {spec_name}");
